#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: GDSN-lib.pl,v 1.5.2.5 2003/04/22 12:06:01 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  Include the general subroutines from web-lib.pl and do init_config().
  do '/usr/webtool/web-lib.pl';
  &init_config();

  use Net::SSLeay qw(post_https make_form);
  use Sys::Hostname;

  ### ##########################################################################
  ### Global Variables
  ### ##########################################################################
  $INCOMING_DIR         =  '/var/cache/webtool/incoming';
  $KEYGEN               =  '/usr/bin/ssh-keygen';
  $LOCAL_MEDIA_DIR      =  '/mnt/cdrom';
  $MD5SUM               =  '/usr/bin/md5sum';
  $OPENSSL              =  '/usr/bin/openssl';
  $PAYLOAD_QUERY_STRING =  '/cgi-bin/gdsn_proxy.cgi';
  $SSH_HOST_KEY_PUB     =  '/etc/ssh/ssh_host_key.pub';
  $UPDATES_HOST         =  'updates.guardiandigital.com';
  $LIDS_CONFIG_SCRIPT   =  '/usr/sbin/config_lids.pl';
  $HOSTNAME             = hostname();
  $DISTRO_STRING        = 'feint';

  %SEVERITY_MAP         = ( '10' => 'Product Enhancement',
                            '20' => 'Updated Version',
                            '30' => 'Minor Bugfix',
                            '40' => 'Major Bugfix',
                            '50' => 'Security Update',
                            '60' => 'New Package' );

  %CONNECTION_ERRORS    = (
    '-10'  => 'Please configure an account number.',
    '-20'  => 'No secret present on local system.',
    '-30'  => 'Invalid Activation Code (Bad Format).',
    '-40'  => 'Invalid Secret (Bad Format).',
    '-50'  => 'Please configure a password.',

    '-3' => 'Could not read payload file.',
    '-2' => 'Could not contact updates host.',
    '-1' => 'Could not write payload file.',
    '1'  => 'Account disabled.  Please contact Guardian Digital.',
    '2'  => 'Activation Code disabled.  Please contact Guardian Digital.',
    '3'  => 'Activation Code expired. Please purchase using the "Support" link below.',
    '4'  => undef,
    '5'  => 'Invalid Activation Code',
    '6'  => 'Invalid Password',
    '5'  => 'Invalid Activation Code.',
    '7'  => 'Invalid Secret.  Please contact Guardian Digital Support.',
    '8'  => 'Invalid Package Name for Retreival.',
    '9'  => 'Invalid Request.  Please try again.',
  );

  ### ##########################################################################
  sub CheckForCDROMDrive() {

    ###  Check 1:  Make sure $LOCAL_MEDIA_DIR exists.
    if (! -d $LOCAL_MEDIA_DIR) {
      &PrintError("The local media directory '$LOCAL_MEDIA_DIR' does not exist.");
      exit;
    }


    ###  Check 2:  Make sure we have a CD-ROM in /etc/fstab.
    my ($HAVE_CDROM); $HAVE_CDROM = 0;
    open FSTAB, "/etc/fstab";
      while (my $line = <FSTAB>) {
        if ($line =~ m/^[^#].*$LOCAL_MEDIA_DIR/) { $HAVE_CDROM = 1; }
      }
    close FSTAB;

    if (! $HAVE_CDROM) {
      &PrintError("Could not find a CD-ROM to mount in '/etc/fstab.'"); exit;
    }
  }


  ### ##########################################################################
  sub MountCDROMDrive() {
    my ($rc); $rc = undef;

    ###  Only attempt to mount the CD-ROM drive if it's not already mounted.
    $rc = system "grep $LOCAL_MEDIA_DIR /proc/mounts > /dev/null 2>&1";
    if ($rc != 0) {
      $rc = system "mount $LOCAL_MEDIA_DIR > /dev/null 2>&1";
      if ($rc != 0) { &PrintError("Could not mount CD-ROM drive."); exit; }
    }

    return $rc;
  }


  ### ##########################################################################
  sub UmountCDROMDrive() {
    my ($rc); $rc = undef;

    ###  Only attempt to umount the CD-ROM drive if it's already mounted.
    $rc = system "grep $LOCAL_MEDIA_DIR /proc/mounts > /dev/null 2>&1";
    if ($rc == 0) {
      $rc = system "umount $LOCAL_MEDIA_DIR > /dev/null 2>&1";
      # if ($rc != 0) { &PrintError("Could not umount CD-ROM drive."); exit; }
    }

    return $rc;
  }


  ### ##########################################################################
  sub fingerprint() { 
    my ($file, $fprint);
  
    $file = shift;
  
    open KEYGEN, "$KEYGEN -l -f $file |";
      ($fprint)  =  (<KEYGEN> =~ m/^[0-9]+\ ([A-Za-z0-9\:]+)\ /);
    close KEYGEN;
  
    return $fprint;
  }

  ### ##########################################################################
  sub GDSN_Get_Payload() {
    ($page) = post_https("$UPDATES_HOST", 443, "/$PAYLOAD_QUERY_STRING", '',
                         make_form('activation_code'  => "$_[0]",
                                   'password'         => "$_[1]",
                                   'secret'           => "$_[2]",
                                   'distro'           => "$DISTRO_STRING",
                                   'hostname'         => "$HOSTNAME",
                                   'fcn'              => "payload"));

    ###  Return enumerated server error if we get one.
    if (! $page)                            { return -2; }
    elsif ($page =~ m/^ERROR\:\ ([0-9]+)$/) { return $1; }

    ###  Now write the payload to a local file.
    open FILE, ">$INCOMING_DIR/Payload.pl" or return -1;
       print FILE $page;
    close FILE;

    return 0;
  }

  ### ##########################################################################
  sub GDSN_Get_Package() {
    ($page) = post_https("$UPDATES_HOST", 443, "/$PAYLOAD_QUERY_STRING", '',
                         make_form('activation_code'  => "$_[0]",
                                   'password'         => "$_[1]",
                                   'secret'           => "$_[2]",
                                   'distro'           => "$DISTRO_STRING",
                                   'hostname'         => "$HOSTNAME",
                                   'package'          => "$_[3]",
                                   'fcn'              => "getpack"));

    ###  Return enumerated server error if we get one.
    if (! $page)                            { return -2; }
    elsif ($page =~ m/^ERROR\:\ ([0-9]+)$/) { return $1; }

    ###  Now write the payload to a local file.
    my ($tmpfile); ($tmpfile) = ($_[3] =~ m/\/([^\/]+)$/);

    open FILE, ">$INCOMING_DIR/$tmpfile";
       print FILE $page;
    close FILE;


    ###  If all went well, return the temporary filename.
    return $tmpfile;
  }

  ### ##########################################################################
  sub SignatureIsValid() {
    my ($out); $out  = `rpm --checksig $_[0]`; chomp $out;

    if ($out =~ m/\ md5\ gpg\ OK$/) { return 1; }
    else                            { return 0; }
  }

  ### ##########################################################################
  sub isNewer() {
    my ($a, $b, $a1, $b1, $c1, $x1, $y1, $z1);

    ($a, $b)         =  @_;
    ($a1, $b1, $c1)  =  split /\./, $a;
    ($x1, $y1, $z1)  =  split /\./, $b;

    if (my $tmp1 = ($x1 <=> $a1))    { return $tmp1; }
    elsif (my $tmp2 = ($y1 <=> $b1)) { return $tmp2; }
    elsif (my $tmp3 = ($z1 <=> $c1)) { return $tmp3; }
    else { return 0; }
  }


  ### ##########################################################################
  sub PrintError {
    my ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>  
ErrorEOF
  }

  ### ##########################################################################
  1;

